<?php
// jcode_mini.phps - PHP
//
// ۸ http://www.spencernetwork.org/
//


function JcodeConvert($str, $from, $to)
{
    //0:AUTO DETECT
    //1:EUC-JP
    //2:Shift_JIS
    //3:ISO-2022-JP(JIS)
    
    if ($from == 0) $from = AutoDetect($str);
    if ($from == 1 && $to == 2) return EUCtoSJIS($str);
    if ($from == 1 && $to == 3) return EUCtoJIS($str);
    if ($from == 2 && $to == 1) return SJIStoEUC($str);
    if ($from == 2 && $to == 3) return SJIStoJIS($str);
    if ($from == 3 && $to == 1) return JIStoEUC($str);
    if ($from == 3 && $to == 2) return JIStoSJIS($str);
    
    return $str;
}

function AutoDetect($str)
{
    //0:US-ASCII
    //1:EUC-JP
    //2:Shift_JIS
    //3:ISO-2022-JP(JIS)
    //(4:Not available)
    //5:Unknown
    
    $b = unpack("C*", $str);
    $n = count($b);
    $is_ascii = TRUE;
    for ($i = 1; $i < $n; $i++) {
        if ($b[$i] == 0x1B && $b[$i+1] == 0x24) return 3;  //JIS
        if ($b[$i] <= 0x7F) continue;  //ASCII
        $is_ascii = FALSE;
        if ($b[$i] <= 0x9F) return 2;  //SJIS
        if ($b[$i] <= 0xDF) return 1;  //EUC
        if ($b[$i] >= 0xF0) return 2;  //SJIS
        if ($b[$i+1] <= 0xA0) return 2;  //SJIS
        if ($b[$i+1] >= 0xFD) return 1;  //EUC
        $i++;
    }
    if ($is_ascii) return 0;
    
    return 5;
}


function JIStoEUC($str_JIS)
{
    $str_EUC = '';
    $mode = 0;
    $b = unpack("C*", $str_JIS);
    $n = count($b);

    for ($i = 1; $i <= $n; $i++) {
        if ($b[$i] == 0x1B) {
            if (($b[$i+1] == 0x24 && $b[$i+2] == 0x42) || ($b[$i+1] == 0x24 && $b[$i+2] == 0x40)) {
                $mode = 1;
            } elseif (($b[$i+1] == 0x28 && $b[$i+2] == 0x49)) {
                $mode = 2;
            } else {
                $mode = 0;
            }
            $i += 3;
        }

        if (!isset($b[$i])) break;
        
        if ($mode == 1) {
            $b[$i] += 0x80; $b[$i+1] += 0x80;
            $str_EUC .= pack("CC", $b[$i], $b[$i+1]);
            $i++;
        } elseif ($mode == 2) {
            $b[$i] += 0x80;
            $str_EUC .= pack("CC", 0x8E, $b[$i]);
        } else {
            $str_EUC .= pack("C", $b[$i]);
        }
    }
    return $str_EUC;
}


function SJIStoEUC($str_SJIS)
{
    $str_EUC = '';
    $b = unpack("C*", $str_SJIS);
    $n = count($b);
    
    for ($i = 1; $i <= $n; $i++) {
        if (0xA1 <= $b[$i] && $b[$i] <= 0xDF) {
            $str_EUC .= pack("CC", 0x8E, $b[$i]);
        } elseif ($b[$i] >= 0x81) {
            $b[$i] <<= 1;
            if ($b[$i+1] < 0x9F) {
                if ($b[$i] < 0x13F) $b[$i] -= 0x61; else $b[$i] -= 0xE1;
                if ($b[$i+1] > 0x7E) $b[$i+1] += 0x60; else $b[$i+1] += 0x61;
            } else {
                if ($b[$i] < 0x13F) $b[$i] -= 0x60; else $b[$i] -= 0xE0;
                $b[$i+1] += 0x02;
            }
            $str_EUC .= pack("CC", $b[$i], $b[$i+1]);
            $i++;
        } else {
            $str_EUC .= pack("C", $b[$i]);
        }
    }
    return $str_EUC;
}

function EUCtoSJIS(&$str_EUC)
{
    $str_SJIS = '';
    $b = unpack("C*", $str_EUC);
    $n = count($b);

    for ($i = 1; $i <= $n; $i++) {
        $b1 = $b[$i];
        if ($b1 == 0x8E) {
            $str_SJIS .= chr($b[$i+1]);
            $i++;
        } elseif ($b1 >= 0x8E) {
            $b2 = $b[$i+1];
            if ($b1 & 0x01) {
                $b1 >>= 1;
                if ($b1 < 0x6F) $b1 += 0x31; else $b1 += 0x71;
                if ($b2 > 0xDF) $b2 -= 0x60; else $b2 -= 0x61;
            } else {
                $b1 >>= 1;
                if ($b1 <= 0x6F) $b1 += 0x30; else $b1 += 0x70;
                $b2 -= 0x02;
            }
            $str_SJIS .= chr($b1).chr($b2);
            $i++;
        } else {
            $str_SJIS .= chr($b1);
        }
    }

    return $str_SJIS;
}


?>
